"""
Pomodoro timer class
"""

from abc import ABC, abstractmethod
import time
from yapona.dbus import DBus


class State(ABC):

    @abstractmethod
    def step(self):
        pass

    @abstractmethod
    def done(self):
        pass


class IdleState(State):

    def __init__(self, dbus):
        self.dbus = dbus
        self.dbus.call("Waiting")

    def step(self):
        pass

    def done(self):
        return False


class RunningState(State):
    def __init__(self, dbus):
        self.dbus = dbus
        self.start_time = time.time()
        self.cur_time = self.start_time
        self.max_duration = 60 * 20

    def step(self):
        self.cur_time = time.time()
        delta = int(self.cur_time - self.start_time)
        if delta < 60:
            msg = f"{delta}s"
        else:
            msg = f"{delta // 60}m:{delta % 60}s"
        self.dbus.call(msg)

    def done(self):
        return self.cur_time - self.start_time >= self.max_duration


class Timer:

    def __init__(self):
        self.dbus = DBus()
        self.state = IdleState(self.dbus)
        self.msg = ""

    def start(self) -> bool:
        if self.state.__class__ != IdleState:
            return False
        self.state = RunningState(self.dbus)
        return True

    def interrupt(self) -> bool:
        if self.state.__class__ != RunningState:
            return False
        self.state = IdleState(self.dbus)
        return True

    def on_done(self):
        if self.state.__class__ == RunningState:
            self.msg = "Pomodoro done. Hooray!"
            self.state = IdleState(self.dbus)

    def update(self):
        self.state.step()
        if self.state.done():
            self.on_done()



import atexit
from yapona.timer import Timer
import os
from threading import Thread, Lock
import time
from gi.repository import GLib
from gi.repository import Notify
from gi.repository import GObject
from gi.repository import AppIndicator3 as appindicator
from gi.repository import Gtk as gtk
import gi
import signal


def get_icon(name):
    icons_dir = os.path.dirname(os.path.realpath(__file__))
    path = os.path.realpath(os.path.join(icons_dir, name))
    assert os.path.exists(path)
    return path


def send_notification(message):
    Notify.init("pomodoro-indicator")
    Notify.Notification.new(message).show()
    Notify.uninit()


class App:

    def __init__(self):
        self.name = "Yapona"
        self.indicator = appindicator.Indicator.new(
            "appindicator", get_icon("focus.svg"),
            appindicator.IndicatorCategory.APPLICATION_STATUS)
        self.indicator.set_status(appindicator.IndicatorStatus.ACTIVE)
        self.indicator.set_menu(self.create_menu())
        self.indicator.set_label(self.name, self.name)
        self.mutex = Lock()
        self.start_time = time.time()
        self.timer = Timer()
        self.update = Thread(target=self.show_seconds)
        self.update.daemon = True
        self.update.start()

    def create_menu(self):
        menu = gtk.Menu()
        item_start = gtk.MenuItem(label="Start")
        item_start.connect("activate", self.start)
        item_reset = gtk.MenuItem(label="Reset")
        item_reset.connect("activate", self.reset)
        item_quit = gtk.MenuItem(label="Quit")
        item_quit.connect("activate", self.quit)
        menu.append(item_start)
        menu.append(item_reset)
        menu.append(item_quit)
        menu.show_all()
        return menu

    def show_seconds(self):
        while True:
            with self.mutex:
                self.timer.update()
                if self.timer.msg:
                    send_notification(self.timer.msg)
                    self.timer.msg = ""
            time.sleep(1)

    def start(self, widget):
        with self.mutex:
            send_notification("Pomodoro started")
            self.timer.start()

    def reset(self, widget):
        with self.mutex:
            send_notification("Pomodoro reset")
            self.timer.interrupt()

    def quit(self, widget):
        gtk.main_quit()


def handle_exit(*args):
    from yapona.dbus import DBus
    bus = DBus()
    bus.call("Waiting")


def main():
    gi.require_version('Gtk', '3.0')
    gi.require_version('Notify', '0.7')
    gi.require_version("AppIndicator3", "0.1")
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    atexit.register(handle_exit)
    signal.signal(signal.SIGTERM, handle_exit)
    app = App()
    gtk.main()


if __name__ == '__main__':
    main()



from dbus_next.glib import MessageBus
from dbus_next.message import Message


def main():
    bus = MessageBus()
    bus.connect_sync()
    message = Message("i3.status.rs",
                      "/Pomodoro",
                      "i3.status.rs",
                      "SetStatus",
                      signature="s", body=["hello"])
    bus.call_sync(message)


if __name__ == "__main__":
    main()



from dbus_next.glib import MessageBus
from dbus_next.message import Message


class DBus:

    def __init__(self):
        self.bus = MessageBus()
        self.bus.connect_sync()

    def call(self, text):
        message = Message("i3.status.rs",
                          "/Pomodoro",
                          "i3.status.rs",
                          "SetStatus",
                          signature="s",
                          body=[text])
        self.bus.call_sync(message)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yapona']

package_data = \
{'': ['*']}

install_requires = \
['dbus-next>=0.2.3,<0.3.0', 'pygobject>=3.42.2,<4.0.0']

entry_points = \
{'console_scripts': ['yapona = yapona.app:main']}

setup_kwargs = {
    'name': 'yapona',
    'version': '0.1.2',
    'description': 'Pomodoro timer',
    'long_description': '# Yapona\n\nYet another Pomodoro Timer...\n',
    'author': 'Dima Lipin',
    'author_email': 'dimich3d@ya.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
